@@ -1,9 +1,10 @@
 # Copyright (c) 2018 Sean Vig
+# Copyright (c) 2022 Aakash Sen Sharma
 
-from pathlib import Path
 import importlib.util
 import os
 import sys
+from pathlib import Path
 
 from cffi import FFI, VerificationError
 from pywayland.ffi_build import ffi_builder as pywayland_ffi
@@ -384,6 +385,53 @@ struct wlr_data_source {
 void wlr_data_source_destroy(struct wlr_data_source *source);
 """
 
+# types/wlr_export_dmabuf_v1.h
+CDEF += """
+struct wlr_export_dmabuf_manager_v1 {
+    struct wl_global *global;
+    struct wl_list frames; // wlr_export_dmabuf_frame_v1::link
+
+    struct wl_listener display_destroy;
+
+    struct {
+        struct wl_signal destroy;
+    } events;
+    ...;
+};
+
+struct wlr_export_dmabuf_frame_v1 {
+    struct wl_resource *resource;
+    struct wlr_export_dmabuf_manager_v1 *manager;
+    struct wl_list link; // wlr_export_dmabuf_manager_v1::frames
+
+    struct wlr_output *output;
+
+    bool cursor_locked;
+
+    struct wl_listener output_commit;
+    ...;
+};
+
+struct wlr_export_dmabuf_manager_v1 *wlr_export_dmabuf_manager_v1_create(
+    struct wl_display *display);
+"""
+
+# types/wlr_viewporter.h
+CDEF += """
+struct wlr_viewporter {
+    struct wl_global *global;
+
+    struct {
+        struct wl_signal destroy;
+    } events;
+
+    struct wl_listener display_destroy;
+    ...;
+};
+
+struct wlr_viewporter *wlr_viewporter_create(struct wl_display *display);
+"""
+
 # types/wlr_foreign_toplevel_management_v1.h
 CDEF += """
 struct wlr_foreign_toplevel_manager_v1 {
@@ -805,6 +853,14 @@ bool pixman_region32_not_empty(struct pixman_region32 *region);
 
 # types/wlr_output.h
 CDEF += """
+struct wlr_output_mode {
+    int32_t width, height;
+    int32_t refresh; // mHz
+    bool preferred;
+    struct wl_list link;
+    ...;
+};
+
 struct wlr_output_state {
     ...;
 };
@@ -1323,6 +1379,12 @@ struct wlr_primary_selection_v1_device_manager *
     wlr_primary_selection_v1_device_manager_create(struct wl_display *display);
 """
 
+# types/wlr_primary_selection.h
+CDEF += """
+void wlr_seat_set_primary_selection(struct wlr_seat *seat,
+    struct wlr_primary_selection_source *source, uint32_t serial);
+"""
+
 # types/wlr_relative_pointer_v1.h
 CDEF += """
 struct wlr_relative_pointer_manager_v1 {
@@ -1582,6 +1644,12 @@ struct wlr_seat_request_set_selection_event {
     ...;
 };
 
+struct wlr_seat_request_set_primary_selection_event {
+    struct wlr_primary_selection_source *source;
+    uint32_t serial;
+    ...;
+};
+
 struct wlr_seat_request_start_drag_event {
     struct wlr_drag *drag;
     struct wlr_surface *origin;
@@ -1861,6 +1929,43 @@ struct wlr_virtual_keyboard_manager_v1* wlr_virtual_keyboard_manager_v1_create(
     struct wl_display *display);
 """
 
+# types/wlr_virtual_pointer_v1.h
+CDEF += """
+struct wlr_virtual_pointer_manager_v1 {
+    struct wl_global *global;
+    struct wl_list virtual_pointers;
+    struct wl_listener display_destroy;
+    struct {
+        struct wl_signal new_virtual_pointer;
+        struct wl_signal destroy;
+    } events;
+    ...;
+};
+
+struct wlr_virtual_pointer_v1 {
+    struct wlr_input_device input_device;
+    struct wl_resource *resource;
+    struct wlr_event_pointer_axis axis_event[2];
+    enum wl_pointer_axis axis;
+    bool axis_valid[2];
+    struct wl_list link;
+    struct {
+        struct wl_signal destroy;
+    } events;
+    ...;
+};
+
+struct wlr_virtual_pointer_v1_new_pointer_event {
+    struct wlr_virtual_pointer_v1 *new_pointer;
+    struct wlr_seat *suggested_seat;
+    struct wlr_output *suggested_output;
+    ...;
+};
+
+struct wlr_virtual_pointer_manager_v1* wlr_virtual_pointer_manager_v1_create(
+    struct wl_display *display);
+"""
+
 # types/wlr_xcursor_manager.h
 CDEF += """
 struct wlr_xcursor_manager *wlr_xcursor_manager_create(const char *name,
@@ -1871,6 +1976,26 @@ int wlr_xcursor_manager_load(struct wlr_xcursor_manager *manager,
     float scale);
 void wlr_xcursor_manager_set_cursor_image(struct wlr_xcursor_manager *manager,
     const char *name, struct wlr_cursor *cursor);
+struct wlr_xcursor *wlr_xcursor_manager_get_xcursor(
+    struct wlr_xcursor_manager *manager, const char *name, float scale);
+"""
+
+# xcursor.h
+CDEF += """
+struct wlr_xcursor_image {
+    uint32_t width;
+    uint32_t height;
+    uint32_t hotspot_x;
+    uint32_t hotspot_y;
+    uint32_t delay;
+    uint8_t *buffer;
+};
+struct wlr_xcursor {
+    unsigned int image_count;
+    struct wlr_xcursor_image **images;
+    char *name;
+    uint32_t total_delay;
+};
 """
 
 # types/wlr_xdg_decoration_v1.h
@@ -2217,6 +2342,8 @@ void wlr_xdg_surface_for_each_surface(struct wlr_xdg_surface *surface,
 
 void wlr_xdg_surface_for_each_popup_surface(struct wlr_xdg_surface *surface,
     wlr_surface_iterator_func_t iterator, void *user_data);
+
+uint32_t wlr_xdg_surface_schedule_configure(struct wlr_xdg_surface *surface);
 """
 
 # util/edges.h
@@ -2283,10 +2410,12 @@ SOURCE = """
 #include <wlr/backend/libinput.h>
 #include <wlr/render/allocator.h>
 #include <wlr/render/wlr_renderer.h>
-#include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_compositor.h>
+#include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_data_control_v1.h>
 #include <wlr/types/wlr_data_device.h>
+#include <wlr/types/wlr_export_dmabuf_v1.h>
+#include <wlr/types/wlr_viewporter.h>
 #include <wlr/types/wlr_foreign_toplevel_management_v1.h>
 #include <wlr/types/wlr_gamma_control_v1.h>
 #include <wlr/types/wlr_idle.h>
@@ -2302,6 +2431,7 @@ SOURCE = """
 #include <wlr/types/wlr_output_management_v1.h>
 #include <wlr/types/wlr_output_power_management_v1.h>
 #include <wlr/types/wlr_pointer_constraints_v1.h>
+#include <wlr/types/wlr_primary_selection.h>
 #include <wlr/types/wlr_primary_selection_v1.h>
 #include <wlr/types/wlr_relative_pointer_v1.h>
 #include <wlr/types/wlr_scene.h>
@@ -2310,6 +2440,7 @@ SOURCE = """
 #include <wlr/types/wlr_seat.h>
 #include <wlr/types/wlr_server_decoration.h>
 #include <wlr/types/wlr_virtual_keyboard_v1.h>
+#include <wlr/types/wlr_virtual_pointer_v1.h>
 #include <wlr/types/wlr_xcursor_manager.h>
 #include <wlr/types/wlr_xdg_decoration_v1.h>
 #include <wlr/types/wlr_xdg_output_v1.h>
